/*
 * digits.c
 *
 * Eine einfache Zahlendarstelung auf dem Ping-Pong Board.
 *
 * Kompilierbar mittels AVR Studio 4 oder WinAVR 
 *
 * Der Sourcecode und das Hexfile drfen frei verwendet werden.
 * Nutzung erfolgt auf eigene Gefahr.
 *
 * Ver.     Date         Author           Comments
 * -------  ----------   --------------   ------------------------------
 * 1.00	    25.12.2009   Piotr Platek     initial 
 */


/* -----------------------------------------
 * Defines (Prprozessor Makros)
 * -----------------------------------------*/

#define F_CPU 8000000UL	                       /* CPU Takt (fr delay-Routine) */


/* -----------------------------------------
 * Includes
 * -----------------------------------------*/
#include <stdlib.h>
#include <inttypes.h>		                   /* Definition der Datentypen uint8_t usw. */
#include <avr/interrupt.h>                     /* Interruptbehandlungsroutinen (fr Timerinterrupt) */
#include <avr/sleep.h>
#include <avr/io.h>
#include <util/delay.h>		                   /* Definition der Verzgerungsfunktionen (_delay_ms) */
#include <avr/pgmspace.h>                      /* Hilfsfunktionen um Daten aus dem Flash zu lesen */
#include "fonts.h"
#include "display.h"                              /* Definition des Zeichensatzes */


/* ----------------------------------------------------------------------------
 * Global variables
 */


/* ----------------------------------------------------------------------------
 * Function prototypes
 */
//void LedScroll(char *c, int8_t lx, int8_t rx, int8_t y, uint16_t t);
//void LedPrint(uint8_t *c, int8_t x, int8_t y);
void LedCLS();


/* -------------------------------------------------------------------------
 * Main Funktion
 *
 * Initialisiert den Timer Interrupt und
 * behandelt die Laufschrift
 * -------------------------------------------------------------------------*/
int main(void)
{
	
	
	uint8_t ii;

	cli();                              // Interrupts sperren (damit keiner dazwischenfunkt)

	/*---------------------------------------------------
	 * Ports konfigurieren (Ein-/Ausgnge)
	 *---------------------------------------------------*/
	DDRC = 0x0f;   //  0x0f PORTC als AD-Eingang
	DDRB = 0xff;   //  Portb = Output
	DDRD = 0xff;   //  Portd = Output

	/*---------------------------------------------------------------------------
	 * 8-Bit Timer TCCR0 fr das Multiplexing der LEDs initialisieren
	 * Es wird ca. alle 2 Mikrosekunden ein Overflow0 Interrupt ausgelst
	 * Berechnung: T = Vorteiler * Wertebereich Zhler / Taktfreuenz
	 * = 64 * 256 / ( 8000000 Hz ) = 2,048 ms
	 *---------------------------------------------------------------------------*/
	TCCR0 |= _BV(CS01) | _BV(CS00);		// 8-bit Timer mit 1/64 Vorteiler
	TIFR  |= _BV(TOV0); 				// Clear overflow flag (TOV0)
	TIMSK |= _BV(TOIE0); 				// timer0 will create overflow interrupt
	

	sei();							    // Interrupts erlauben

	/*---------------------------------------------------
	 * Hauptschleife (Laufschrift erzeugen)
	 *---------------------------------------------------*/
	
/*
	leds[0] = 0b1000000001;
	leds[1] = 0b0100000001;
	leds[2] = 0b0010000000;
	leds[3] = 0b0001000000;
	leds[4] = 0b0000100000;
	leds[5] = 0b0000010000;
	leds[6] = 0b0000001000;
	leds[7] = 0b0000000100;
	leds[8] = 0b0000000010;
	leds[9] = 0b0000000001;
	leds[10]= 0b0000000001;
	leds[11]= 0b1111111111;

	while (1);
*/

	
	char p6[10] = ":\0";
	char p7[10] = ":\0";
	


//	ADMUX = _BV(ADLAR) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0);
	ADMUX = _BV(MUX2) | _BV(MUX1) | _BV(MUX0);
//	ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0) | _BV(ADFR) ;
	ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
	ADCSRA |= _BV(ADSC);

	ii =0b0111;
	

	PotiC = 1;
	LPotiAvr[0] = 0;
	RPotiAvr[0] = 0;

//while (1)
//{
//	LedScroll("___<<<JUSTYNA>>>___ TO ... NAJLEPSZA CORKA!           DU BIST DIE BESTE DOCHTER DER WELT        ", 0, 11, 3, 90);
//	LedPrint("012", 0, 1);
//}

//	GameStart();

	while (1)
	{

		if ( !(ADCSRA & _BV(ADSC)) )
		{ 
			if ( ii == 0b0111)
			{
				LPotiAvr[PotiC] = ADC;
				ADMUX &= ~(_BV(MUX2) | _BV(MUX1) | _BV(MUX0));	
				ADMUX |= _BV(MUX2) | _BV(MUX1);
				ii = 0b0110;
				// ADCSRA |= _BV(ADSC);
			}
			else
			{
				RPotiAvr[PotiC++] = ADC;
				ADMUX &= ~(_BV(MUX2) | _BV(MUX1));	
				ADMUX |= _BV(MUX2) | _BV(MUX1) | _BV(MUX0);
				ii =0b0111;
				// ADCSRA |= _BV(ADSC);
				
			}
			ADCSRA |= _BV(ADSC); // start new conversion
			if (PotiC >= PotiMaxC) 
			{
				LPotiAvr[0] = 0;
				RPotiAvr[0] = 0;
				for (PotiC = 1; PotiC < PotiMaxC; PotiC++)
				{
					LPotiAvr[0] += LPotiAvr[PotiC];
					RPotiAvr[0] += RPotiAvr[PotiC];
				}
				LPotiAvr[0] /= (PotiMaxC-1);
				RPotiAvr[0] /= (PotiMaxC-1);
				PotiC = 1;
			}
		}


		if ( LPotiAvr[0] > 512) StandBy();


		LedPrint(itoa(RPotiAvr[0], p6, 10), 0, 5);
		LedPrint(itoa(LPotiAvr[0], p7, 10), 0, 0);

//		LedScroll(p6, 0, 3, 5, 250);
//		LedScroll(p7, 4, 7, 0, 250);

//		_delay_ms(100);	
//		LedPrint("   ", 0, 5);
//		LedPrint("   ", 0, 0);

	}

/*
    int16_t d;	
    int8_t str[10] = "\0";

	while(1)                                              // Endlosschleife
	{
	
		for (d=0; d < 3; d++)
		{
			LedPrint(itoa(d, str, 10),0,0);
			_delay_ms(1000);
			LedPrint("<<<<",0,0);
		}
	}

*/
	return 0;
	
	
}

void GameStart()
{
	uint8_t str[4];

	LedCLS();
	LedPrint(strcpy_P(str, str_00), 0, 3);
	_delay_ms(3000);
	LedCLS();
}

void LedCLS()
{
	for (int8_t i = 0; i < SWIDTH; leds[i++] = 0);
}


void LedPrint(uint8_t *c, int8_t x, int8_t y)
{
  uint8_t i, fn, FontWidth;
  uint16_t 	ResetColV;
  uint8_t *text;

  ResetColV = ResetCol << (FONTHEIGHT-y);  
  for (text=c; (*text); text++)
  {
	fn = *text - FirstASCIICharacter;
	FontWidth = pgm_read_byte(&font[fn][FONTWIDTHIDX])+FONTDEFIDX;
	for (i=FONTDEFIDX ; i < FontWidth ; i++)
	{
        if ((x >= 0) && (x < SWIDTH))
		{
		  leds[x] &= ~ResetColV;
		  leds[x] |= pgm_read_byte(&font[fn][i]) << (FONTHEIGHT-y);
		}
		x++;
	}
	x++;  // immer eine Spalte leer
  }
}

void LedScroll(char *c, int8_t lx, int8_t rx, int8_t y, uint16_t t)
{
  
  uint8_t fcol, scol, fn, FontWidth;
  uint16_t ResetColV;
  char *text;

  
  if ( (rx >= 0) && (rx < SWIDTH) && (lx >= 0) && (lx < SWIDTH) && (lx < rx) )  // scroll only in display and for xl < xr
  {
	ResetColV = ResetCol << (FONTHEIGHT-y);  
	for (text=c; (*text); text++)
	{
		fn = *text - FirstASCIICharacter;
		FontWidth = pgm_read_byte(&font[fn][FONTWIDTHIDX])+FONTDEFIDX;
		for (fcol=FONTDEFIDX; fcol < FontWidth; fcol++) 
		{    
			for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left
			{
				leds[scol] &= ~ResetColV;
				leds[scol] |= leds[scol+1] & ResetColV;
			}
			leds[rx] &= ~ResetColV;
			leds[rx] |= pgm_read_byte_near(&font[fn][fcol])<<(FONTHEIGHT-y);
			_delay_ms(t);	
		}
		for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left after one char
		{
			leds[scol] &= ~ResetColV;
			leds[scol] |= leds[scol+1] & ResetColV;
		}
		leds[rx] &= ~ResetColV;  // and add one empty column
	}
  }
}



/*

void LedScroll(char *c, int8_t lx, int8_t rx, int8_t y, uint16_t t)
{
  
  uint8_t fcol, scol, fn, FontWidth;
  uint16_t ResetColV;
  char *text;

  
  if ( (rx >= 0) && (rx < WIDTH) && (lx >= 0) && (lx < WIDTH) && (lx < rx) )  // scroll only in display and for xl < xr
  {
	ResetColV = ResetCol << (HFH-y);  
	for (text=c; (*text); text++)
	{
		fn = *text - '0';
		FontWidth = pgm_read_byte_near(&font[fn][0]);
		for (fcol=1 ; fcol <= FontWidth; fcol++) 
		{    
			for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left
			{
				leds[scol] &= ~ResetColV;
				leds[scol] |= leds[scol+1] & ResetColV;
			}
			leds[rx] &= ~ResetColV;
			leds[rx] |= pgm_read_byte_near(&font[fn][fcol])<<(HFH-y);
			_delay_ms(t);	
		}
		for (scol = lx ; scol < rx ; scol++)  // scroll current text one column to the left after one char
		{
			leds[scol] &= ~ResetColV;
			leds[scol] |= leds[scol+1] & ResetColV;
		}
		leds[rx] &= ~ResetColV;  // and add one empty column
	}
  }
}

void LedPrint(uint8_t *c, int8_t x, int8_t y)
{
  uint8_t i, fn, FontWidth;
  uint16_t 	ResetColV;
  uint8_t *text;

  ResetColV = ResetCol << (HFH-y);  
  for (text=c; (*text); text++)
  {
	fn = *text - '0';
	FontWidth = pgm_read_byte(&font[fn][0]);
	for (i=1 ; i <= FontWidth ; i++) {
        if ((x >= 0) && (x < WIDTH)) {
		  leds[x]&=~ResetColV;
		  leds[x]|=pgm_read_byte(&font[fn][i])<<(HFH-y);
		}
		x++;
	}
	x++;  // immer eine Spalte leer
  }
}


*/


/* -------------------------------------------------------------------------
 * Interrupt Routine
 *
 * Gibt nacheinander alle Spalten mit LED-Daten aus.
 * Dazu wird mittels der Schieberegister die aktuelle Spalte
 * ausgewhlt und dann das Bitmuster derselben auf die Ports
 * gegeben.
 * Beim nchsten Interrupt ist dann die nchste Spalte dran.
 * -------------------------------------------------------------------------*/
// interrupt routine
SIGNAL (SIG_OVERFLOW0)
{
	uint16_t ledval;
	uint8_t portcout;
	uint8_t portdout;

	cli();							/* Interrupts verbieten */

	/*--------------------------------------------------
	 * Aktuelle Spalte ermitteln
	 *--------------------------------------------------*/
	if (col == 12)
	{
		col = 0;
	}

	/*--------------------------------------------------
	 * Ports initialisieren
	 *--------------------------------------------------*/
	PORTD = 0;
	PORTB = 0;
	PORTC = 0;

	/*---------------------------------------------------
	 * Eine einzelne 0 durch die Schiebergister schieben
	 *---------------------------------------------------*/
	if ( col == 0 )
	{
		PORTB &= ~(1 << 4);        /* Bei der ersten Spalte eine 0 ausgeben (PB4 = 0) */
		                            /* Diese 0 geht auf die Reise durch die Schieberegister */
	}
	else
	{
		PORTB |= (1 << 4);         /* Danach Einsen hinterherschicken (PB4 = 1) */
	}

	/*---------------------------------------------------
	 * Impulse fr die Schieberegister generieren
	 *---------------------------------------------------*/
	PORTB |= (1 << 3);             /* PB3 = 1 (cl) */
	PORTB &= ~(1 << 3);            /* PB3 = 0 (!cl) */

	PORTB |= (1 << 2);             /* PB2 = 1 (str) */
	PORTB &= ~(1 << 2);            /* PB2 = 0 (!str) */

	/*---------------------------------------------------
	 * Daten der Spalte holen und auf die Ports verteilen
	 *---------------------------------------------------*/
	ledval = leds[col];
	portdout = ledval & 0xff;      /* low byte */
	portcout = portdout & 0x0f;    /* low nibble */
	portdout = portdout & 0xf0;    /* high nibble */

	PORTD = portdout & 0xff;
	PORTC = portcout & 0xff;
	PORTB = (ledval >> 8) & 0x03;  /* high byte */
	
	col++;

	sei();						   /* Interrupts wieder erlauben */
}

